@@ -5,11 +5,9 @@
 import io
 import os
 import time
-import re
 import copy
 import datetime
 import warnings
-import importlib
 from pkg_resources import parse_version
 
 import numpy as np
@@ -25,8 +23,11 @@ from . import util
 from . import version
 from . import versioning
 from . import yamlutil
-from .exceptions import AsdfDeprecationWarning
+from . import _display as display
+from .exceptions import AsdfDeprecationWarning, AsdfWarning, AsdfConversionWarning
 from .extension import AsdfExtensionList, default_extensions
+from .util import NotSet
+from .search import AsdfSearchResult
 
 from .tags.core import AsdfObject, Software, HistoryEntry, ExtensionMetadata
 
@@ -109,23 +110,30 @@ class AsdfFile(versioning.VersionedMixin):
             standard.
 
         """
+        self._extensions = []
+        self._extension_metadata = {}
+        self._process_extensions(extensions)
 
         if custom_schema is not None:
-            self._custom_schema = schema.load_custom_schema(custom_schema)
-            schema.check_schema(self._custom_schema)
+            self._custom_schema = schema._load_schema_cached(custom_schema, self.resolver, True, False)
         else:
             self._custom_schema = None
 
-        self._extensions = []
-        self._extension_metadata = {}
-
-        self._process_extensions(extensions)
         self._ignore_version_mismatch = ignore_version_mismatch
         self._ignore_unrecognized_tag = ignore_unrecognized_tag
         self._ignore_implicit_conversion = ignore_implicit_conversion
 
+        # Set of (string, string) tuples representing tag version mismatches
+        # that we've already warned about for this file.
+        self._warned_tag_pairs = set()
+
         self._file_format_version = None
 
+        # Context of a call to treeutil.walk_and_modify, needed in the AsdfFile
+        # in case walk_and_modify is re-entered by extension code (via
+        # custom_tree_to_tagged_tree or tagged_tree_to_custom_tree).
+        self._tree_modification_context = treeutil._TreeModificationContext()
+
         self._fd = None
         self._closed = False
         self._external_asdf_by_uri = {}
@@ -134,7 +142,9 @@ class AsdfFile(versioning.VersionedMixin):
             readonly=_readonly)
         self._uri = None
         if tree is None:
-            self.tree = {}
+            # Bypassing the tree property here, to avoid validating
+            # an empty tree.
+            self._tree = AsdfObject()
         elif isinstance(tree, AsdfFile):
             if self._extensions != tree._extensions:
                 raise ValueError(
@@ -180,7 +190,7 @@ class AsdfFile(versioning.VersionedMixin):
                 if strict:
                     raise RuntimeError(fmt_msg)
                 else:
-                    warnings.warn(fmt_msg)
+                    warnings.warn(fmt_msg, AsdfWarning)
 
             elif extension.software:
                 installed = self._extension_metadata[extension.extension_class]
@@ -199,7 +209,7 @@ class AsdfFile(versioning.VersionedMixin):
                     if strict:
                         raise RuntimeError(fmt_msg)
                     else:
-                        warnings.warn(fmt_msg)
+                        warnings.warn(fmt_msg, AsdfWarning)
 
     def _process_extensions(self, extensions):
         if extensions is None or extensions == []:
@@ -224,6 +234,8 @@ class AsdfFile(versioning.VersionedMixin):
         self._extension_metadata.update(default_extensions.package_metadata)
 
     def _update_extension_history(self):
+        if self.version < versioning.NEW_HISTORY_FORMAT_MIN_VERSION:
+            return
 
         if 'history' not in self.tree:
             self.tree['history'] = dict(extensions=[])
@@ -233,7 +245,8 @@ class AsdfFile(versioning.VersionedMixin):
             self.tree['history'] = dict(entries=histlist, extensions=[])
             warnings.warn("The ASDF history format has changed in order to "
                           "support metadata about extensions. History entries "
-                          "should now be stored under tree['history']['entries'].")
+                          "should now be stored under tree['history']['entries'].",
+                          AsdfWarning)
         elif 'extensions' not in self.tree['history']:
             self.tree['history']['extensions'] = []
 
@@ -242,7 +255,7 @@ class AsdfFile(versioning.VersionedMixin):
             ext_meta = ExtensionMetadata(extension_class=ext_name)
             metadata = self._extension_metadata.get(ext_name)
             if metadata is not None:
-                ext_meta.software = dict(name=metadata[0], version=metadata[1])
+                ext_meta['software'] = Software(name=metadata[0], version=metadata[1])
 
             for i, entry in enumerate(self.tree['history']['extensions']):
                 # Update metadata about this extension if it already exists
@@ -416,8 +429,13 @@ class AsdfFile(versioning.VersionedMixin):
         return self._comments
 
     def _validate(self, tree, custom=True, reading=False):
-        tagged_tree = yamlutil.custom_tree_to_tagged_tree(
-            tree, self)
+        if reading:
+            # If we're validating on read then the tree
+            # is already guaranteed to be in tagged form.
+            tagged_tree = tree
+        else:
+            tagged_tree = yamlutil.custom_tree_to_tagged_tree(
+                tree, self)
         schema.validate(tagged_tree, self, reading=reading)
         # Perform secondary validation pass if requested
         if custom and self._custom_schema:
@@ -597,7 +615,8 @@ class AsdfFile(versioning.VersionedMixin):
                    _get_yaml_content=False,
                    _force_raw_types=False,
                    strict_extension_check=False,
-                   ignore_missing_extensions=False):
+                   ignore_missing_extensions=False,
+                   validate_on_read=True):
         """Attempt to populate AsdfFile data from file-like object"""
 
         if strict_extension_check and ignore_missing_extensions:
@@ -625,8 +644,8 @@ class AsdfFile(versioning.VersionedMixin):
             self.version = version
 
         yaml_token = fd.read(4)
-        tree = {}
         has_blocks = False
+        tree = None
         if yaml_token == b'%YAM':
             reader = fd.reader_until(
                 constants.YAML_END_MARKER_REGEX, 7, 'End of YAML marker',
@@ -641,13 +660,20 @@ class AsdfFile(versioning.VersionedMixin):
             # We parse the YAML content into basic data structures
             # now, but we don't do anything special with it until
             # after the blocks have been read
-            tree = yamlutil.load_tree(reader, self, self._ignore_version_mismatch)
+            tree = yamlutil.load_tree(reader)
             has_blocks = fd.seek_until(constants.BLOCK_MAGIC, 4, include=True)
         elif yaml_token == constants.BLOCK_MAGIC:
             has_blocks = True
         elif yaml_token != b'':
             raise IOError("ASDF file appears to contain garbage after header.")
 
+        if tree is None:
+            # At this point the tree should be tagged, but we want it to be
+            # tagged with the core/asdf version appropriate to this file's
+            # ASDF Standard version.  We're using custom_tree_to_tagged_tree
+            # to select the correct tag for us.
+            tree = yamlutil.custom_tree_to_tagged_tree(AsdfObject(), self)
+
         if has_blocks:
             self._blocks.read_internal_blocks(
                 fd, past_magic=True, validate_checksums=validate_checksums)
@@ -657,11 +683,12 @@ class AsdfFile(versioning.VersionedMixin):
         if not do_not_fill_defaults:
             schema.fill_defaults(tree, self, reading=True)
 
-        try:
-            self._validate(tree, reading=True)
-        except ValidationError:
-            self.close()
-            raise
+        if validate_on_read:
+            try:
+                self._validate(tree, reading=True)
+            except ValidationError:
+                self.close()
+                raise
 
         tree = yamlutil.tagged_tree_to_custom_tree(tree, self, _force_raw_types)
 
@@ -680,7 +707,8 @@ class AsdfFile(versioning.VersionedMixin):
                    _get_yaml_content=False,
                    _force_raw_types=False,
                    strict_extension_check=False,
-                   ignore_missing_extensions=False):
+                   ignore_missing_extensions=False,
+                   validate_on_read=True):
         """Attempt to open file-like object as either AsdfFile or AsdfInFits"""
         if not is_asdf_file(fd):
             try:
@@ -695,7 +723,8 @@ class AsdfFile(versioning.VersionedMixin):
                             strict_extension_check=strict_extension_check,
                             ignore_missing_extensions=ignore_missing_extensions,
                             ignore_unrecognized_tag=self._ignore_unrecognized_tag,
-                            _extension_metadata=self._extension_metadata)
+                            _extension_metadata=self._extension_metadata,
+                            validate_on_read=validate_on_read)
             except ValueError:
                 raise ValueError(
                     "Input object does not appear to be an ASDF file or a FITS with " +
@@ -711,7 +740,8 @@ class AsdfFile(versioning.VersionedMixin):
                 _get_yaml_content=_get_yaml_content,
                 _force_raw_types=_force_raw_types,
                 strict_extension_check=strict_extension_check,
-                ignore_missing_extensions=ignore_missing_extensions)
+                ignore_missing_extensions=ignore_missing_extensions,
+                validate_on_read=validate_on_read)
 
     @classmethod
     def open(cls, fd, uri=None, mode='r',
@@ -1034,7 +1064,6 @@ class AsdfFile(versioning.VersionedMixin):
         if version is not None:
             self.version = version
 
-
         with generic_io.get_file(fd, mode='w') as fd:
             # TODO: This is not ideal: we really should pass the URI through
             # explicitly to wherever it is required instead of making it an
@@ -1055,12 +1084,8 @@ class AsdfFile(versioning.VersionedMixin):
         Finds all external "JSON References" in the tree and converts
         them to `reference.Reference` objects.
         """
-        # Since this is the first place that the tree is processed when
-        # creating a new ASDF object, this is where we pass the option to
-        # ignore warnings about implicit type conversions.
         # Set directly to self._tree, since it doesn't need to be re-validated.
-        self._tree = reference.find_references(self._tree, self,
-                ignore_implicit_conversion=self._ignore_implicit_conversion)
+        self._tree = reference.find_references(self._tree, self)
 
     def resolve_references(self, do_not_fill_defaults=False):
         """
@@ -1122,7 +1147,8 @@ class AsdfFile(versioning.VersionedMixin):
             if hook is not None:
                 return hook(node, self)
             return node
-        tree = treeutil.walk_and_modify(self.tree, walker)
+        tree = treeutil.walk_and_modify(self.tree, walker,
+            ignore_implicit_conversion=self._ignore_implicit_conversion)
 
         if validate:
             self._validate(tree)
@@ -1196,16 +1222,30 @@ class AsdfFile(versioning.VersionedMixin):
         if software is not None:
             entry['software'] = software
 
-        if 'history' not in self.tree:
-            self.tree['history'] = dict(entries=[])
+        if self.version >= versioning.NEW_HISTORY_FORMAT_MIN_VERSION:
+            if 'history' not in self.tree:
+                self.tree['history'] = dict(entries=[])
+            elif 'entries' not in self.tree['history']:
+                self.tree['history']['entries'] = []
 
-        self.tree['history']['entries'].append(entry)
+            self.tree['history']['entries'].append(entry)
 
-        try:
-            self.validate()
-        except Exception:
-            self.tree['history']['entries'].pop()
-            raise
+            try:
+                self.validate()
+            except Exception:
+                self.tree['history']['entries'].pop()
+                raise
+        else:
+            if 'history' not in self.tree:
+                self.tree['history'] = []
+
+            self.tree['history'].append(entry)
+
+            try:
+                self.validate()
+            except Exception:
+                self.tree['history'].pop()
+                raise
 
     def get_history_entries(self):
         """
@@ -1228,6 +1268,92 @@ class AsdfFile(versioning.VersionedMixin):
 
         return []
 
+    def info(self, max_rows=display.DEFAULT_MAX_ROWS, max_cols=display.DEFAULT_MAX_COLS, show_values=display.DEFAULT_SHOW_VALUES):
+        """
+        Print a rendering of this file's tree to stdout.
+
+        Parameters
+        ----------
+        max_rows : int, tuple, or None, optional
+            Maximum number of lines to print.  Nodes that cannot be
+            displayed will be elided with a message.
+            If int, constrain total number of displayed lines.
+            If tuple, constrain lines per node at the depth corresponding \
+                to the tuple index.
+            If None, display all lines.
+
+        max_cols : int or None, optional
+            Maximum length of line to print.  Nodes that cannot
+            be fully displayed will be truncated with a message.
+            If int, constrain length of displayed lines.
+            If None, line length is unconstrained.
+
+        show_values : bool, optional
+            Set to False to disable display of primitive values in
+            the rendered tree.
+        """
+        lines = display.render_tree(self.tree, max_rows=max_rows, max_cols=max_cols, show_values=show_values, identifier="root.tree")
+        print("\n".join(lines))
+
+    def search(self, key=NotSet, type=NotSet, value=NotSet, filter=None):
+        """
+        Search this file's tree.
+
+        Parameters
+        ----------
+        key : NotSet, str, or any other object
+            Search query that selects nodes by dict key or list index.
+            If NotSet, the node key is unconstrained.
+            If str, the input is searched among keys/indexes as a regular
+            expression pattern.
+            If any other object, node's key or index must equal the queried key.
+
+        type : NotSet, str, or builtins.type
+            Search query that selects nodes by type.
+            If NotSet, the node type is unconstrained.
+            If str, the input is searched among (fully qualified) node type
+            names as a regular expression pattern.
+            If builtins.type, the node must be an instance of the input.
+
+        value : NotSet, str, or any other object
+            Search query that selects nodes by value.
+            If NotSet, the node value is unconstrained.
+            If str, the input is searched among values as a regular
+            expression pattern.
+            If any other object, node's value must equal the queried value.
+
+        filter : callable
+            Callable that filters nodes by arbitrary criteria.
+            The callable accepts one or two arguments:
+
+            - the node
+            - the node's list index or dict key (optional)
+
+            and returns True to retain the node, or False to remove it from
+            the search results.
+
+        Returns
+        -------
+        asdf.search.AsdfSearchResult
+            the result of the search
+        """
+        result = AsdfSearchResult(["root.tree"], self.tree)
+        return result.search(key=key, type=type, value=value, filter=filter)
+
+    # This function is called from within TypeIndex when deserializing
+    # the tree for this file.  It is kept here so that we can keep
+    # state on the AsdfFile and prevent a flood of warnings for the
+    # same tag.
+    def _warn_tag_mismatch(self, tag, best_tag):
+        if not self._ignore_version_mismatch and (tag, best_tag) not in self._warned_tag_pairs:
+            message = (
+                "No explicit ExtensionType support provided for tag '{}'. "
+                "The ExtensionType subclass for tag '{}' will be used instead. "
+                "This fallback behavior will be removed in asdf 3.0."
+            ).format(tag, best_tag)
+            warnings.warn(message, AsdfConversionWarning)
+            self._warned_tag_pairs.add((tag, best_tag))
+
 
 # Inherit docstring from dictionary
 AsdfFile.keys.__doc__ = dict.keys.__doc__
@@ -1257,7 +1383,8 @@ def open_asdf(fd, uri=None, mode=None, validate_checksums=False,
               ignore_version_mismatch=True, ignore_unrecognized_tag=False,
               _force_raw_types=False, copy_arrays=False, lazy_load=True,
               custom_schema=None, strict_extension_check=False,
-              ignore_missing_extensions=False, _compat=False):
+              ignore_missing_extensions=False, validate_on_read=True,
+              _compat=False):
     """
     Open an existing ASDF file.
 
@@ -1326,6 +1453,10 @@ def open_asdf(fd, uri=None, mode=None, validate_checksums=False,
         contains metadata about extensions that are not available. Defaults
         to `False`.
 
+    validate_on_read : bool, optional
+        When `True`, validate the newly opened file against tag and custom
+        schemas.  Recommended unless the file is already known to be valid.
+
     Returns
     -------
     asdffile : AsdfFile
@@ -1352,7 +1483,8 @@ def open_asdf(fd, uri=None, mode=None, validate_checksums=False,
         do_not_fill_defaults=do_not_fill_defaults,
         _force_raw_types=_force_raw_types,
         strict_extension_check=strict_extension_check,
-        ignore_missing_extensions=ignore_missing_extensions)
+        ignore_missing_extensions=ignore_missing_extensions,
+        validate_on_read=validate_on_read)
 
 
 def is_asdf_file(fd):
