@@ -6,8 +6,10 @@ import warnings
 from .exceptions import AsdfDeprecationWarning
 # This is not exhaustive, but represents the public API
 from .versioning import join_tag_version, split_tag_version
-from .types import (AsdfType, CustomType, format_tag, ExtensionTypeMeta,
-                    _all_asdftypes)
+from .types import (AsdfType, CustomType, format_tag, ExtensionTypeMeta)
+
+__all__ = ["join_tag_version", "split_tag_version", "AsdfType", "CustomType",
+    "format_tag", "ExtensionTypeMeta"]
 
 warnings.warn(
     "The module asdf.asdftypes has been deprecated and will be removed in 3.0. "
