@@ -12,7 +12,6 @@ from collections import namedtuple
 from urllib import parse as urlparse
 
 import numpy as np
-from numpy.ma.core import masked_array
 
 import yaml
 
@@ -20,7 +19,6 @@ from . import compression as mcompression
 from .compat.numpycompat import NUMPY_LT_1_7
 from . import constants
 from . import generic_io
-from . import stream
 from . import treeutil
 from . import util
 from . import yamlutil
@@ -482,7 +480,9 @@ class BlockManager:
 
         yaml_content = content[content.find(b'\n') + 1:]
 
-        offsets = yaml.load(yaml_content,
+        # The following call to yaml.load is safe because we're
+        # using pyyaml's SafeLoader.
+        offsets = yaml.load(yaml_content, # nosec
                             Loader=yamlutil._yaml_base_loader)
 
         # Make sure the indices look sane
@@ -586,9 +586,11 @@ class BlockManager:
             block.output_compression = all_array_compression
 
         auto_inline = getattr(ctx, '_auto_inline', None)
-        if auto_inline:
+        if auto_inline and block.array_storage in ['internal', 'inline']:
             if np.product(block.data.shape) < auto_inline:
                 self.set_array_storage(block, 'inline')
+            else:
+                self.set_array_storage(block, 'internal')
 
     def finalize(self, ctx):
         """
@@ -716,20 +718,6 @@ class BlockManager:
 
         raise ValueError("block not found.")
 
-    def _should_inline(self, array):
-
-        if not np.issubdtype(array.dtype, np.number):
-            return False
-
-        if isinstance(array, masked_array):
-            return False
-
-        # Make sure none of the values are too large to store as literals
-        if (array[~np.isnan(array)] > 2**52).any():
-            return False
-
-        return array.size <= self._inline_threshold_size
-
     def find_or_create_block_for_array(self, arr, ctx):
         """
         For a given array, looks for an existing block containing its
@@ -807,13 +795,7 @@ class Block:
 
     def __init__(self, data=None, uri=None, array_storage='internal',
                  memmap=True, lazy_load=True):
-        if isinstance(data, np.ndarray) and not data.flags.c_contiguous:
-            if data.flags.f_contiguous:
-                self._data = np.asfortranarray(data)
-            else:
-                self._data = np.ascontiguousarray(data)
-        else:
-            self._data = data
+        self._data = data
         self._uri = uri
         self._array_storage = array_storage
 
@@ -873,8 +855,21 @@ class Block:
         return self.offset + self.header_size + self.allocated
 
     def override_byteorder(self, byteorder):
+        """
+        Hook to permit overriding the byteorder value stored in the
+        tree.  This is used to support blocks stored in FITS files.
+        """
         return byteorder
 
+    @property
+    def trust_data_dtype(self):
+        """
+        If True, ignore the datatype and byteorder fields from the
+        tree and take the data array's dtype at face value.  This
+        is used to support blocks stored in FITS files.
+        """
+        return False
+
     @property
     def array_storage(self):
         return self._array_storage
@@ -919,8 +914,10 @@ class Block:
             self._checksum = checksum
 
     def _calculate_checksum(self, data):
-        m = hashlib.new('md5')
-        m.update(self.data.flatten())
+        # The following line is safe because we're only using
+        # the MD5 as a checksum.
+        m = hashlib.new('md5') # nosec
+        m.update(self.data.ravel('K'))
         return m.digest()
 
     def validate_checksum(self):
@@ -1125,7 +1122,9 @@ class Block:
             allocated_size = self.allocated
             used_size = self._size
         self.input_compression = self.output_compression
-        assert allocated_size >= used_size
+
+        if allocated_size < used_size:
+            raise RuntimeError("Block used size {} larger than allocated size {}".format(used_size, allocated_size))
 
         if self.checksum is not None:
             checksum = self.checksum
@@ -1163,7 +1162,8 @@ class Block:
                         used_size=self._size)
                     fd.seek(end)
             else:
-                assert used_size == data_size
+                if used_size != data_size:
+                    raise RuntimeError("Block used size {} is not equal to the data size {}".format(used_size, data_size))
                 fd.write_array(self._data)
 
     @property
