@@ -7,7 +7,7 @@ class AsdfWarning(Warning):
     The base warning class from which all ASDF warnings should inherit.
     """
 
-class AsdfDeprecationWarning(AsdfWarning):
+class AsdfDeprecationWarning(AsdfWarning, DeprecationWarning):
     """
     A warning class to indicate a deprecated feature.
     """
