@@ -6,15 +6,12 @@ import abc
 import warnings
 from pkg_resources import iter_entry_points
 
-import six
-import importlib
-
 from . import types
 from . import resolver
 from .util import get_class_name
 from .type_index import AsdfTypeIndex
 from .version import version as asdf_version
-from .exceptions import AsdfDeprecationWarning
+from .exceptions import AsdfDeprecationWarning, AsdfWarning
 
 
 __all__ = ['AsdfExtension', 'AsdfExtensionList']
@@ -23,8 +20,7 @@ __all__ = ['AsdfExtension', 'AsdfExtensionList']
 ASDF_TEST_BUILD_ENV = 'ASDF_TEST_BUILD'
 
 
-@six.add_metaclass(abc.ABCMeta)
-class AsdfExtension:
+class AsdfExtension(metaclass=abc.ABCMeta):
     """
     Abstract base class defining an extension to ASDF.
     """
@@ -203,7 +199,8 @@ class _DefaultExtensions:
             if not issubclass(ext, AsdfExtension):
                 warnings.warn("Found entry point {}, from {} but it is not a "
                               "subclass of AsdfExtension, as expected. It is "
-                              "being ignored.".format(ext, entry_point.dist))
+                              "being ignored.".format(ext, entry_point.dist),
+                              AsdfWarning)
                 continue
 
             dist = entry_point.dist
