@@ -25,8 +25,6 @@ import http.client
 from urllib import parse as urlparse
 from urllib.request import url2pathname
 
-import six
-
 import numpy as np
 
 from . import util
@@ -97,7 +95,7 @@ size : integer
 
 
 def _array_tofile_chunked(write, array, chunksize):  # pragma: no cover
-    array = array.view(np.uint8).flatten()
+    array = array.view(np.uint8)
     for i in range(0, array.nbytes, chunksize):
         write(array[i:i + chunksize].data)
 
@@ -257,8 +255,7 @@ class _TruncatedReader:
         return content
 
 
-@six.add_metaclass(util.InheritDocstrings)
-class GenericFile:
+class GenericFile(metaclass=util.InheritDocstrings):
     """
     Base class for an abstraction layer around a number of different
     file-like types.  Each of its subclasses handles a particular kind
@@ -359,11 +356,9 @@ class GenericFile:
         time.  The result is a generator where each value is a bytes
         object.
         """
-        i = 0
-        for i in range(0, size - self._blksize, self._blksize):
-            yield self.read(self._blksize)
-        if i < size:
-            yield self.read(size - i)
+        for i in range(0, size, self._blksize):
+            thissize = min(self._blksize, size - i)
+            yield self.read(thissize)
 
     def write(self, content):
         self._fd.write(content)
@@ -377,7 +372,7 @@ class GenericFile:
     """
 
     def write_array(self, array):
-        _array_tofile(None, self.write, array.ravel(order='A'))
+        _array_tofile(None, self.write, array.ravel(order='K'))
 
     def seek(self, offset, whence=0):
         """
@@ -756,7 +751,7 @@ class RealFile(RandomAccessFile):
             arr.flush()
             self.fast_forward(len(arr.data))
         else:
-            _array_tofile(self._fd, self._fd.write, arr.ravel(order='A'))
+            _array_tofile(self._fd, self._fd.write, arr.ravel(order='K'))
 
     def can_memmap(self):
         return True
@@ -1096,7 +1091,7 @@ def get_uri(file_obj):
     ----------
     uri : object
     """
-    if isinstance(file_obj, six.string_types):
+    if isinstance(file_obj, str):
         return file_obj
     if isinstance(file_obj, GenericFile):
         return file_obj.uri
