@@ -32,6 +32,7 @@ is not intended to be exposed to the end user.
 """
 
 from collections import UserDict, UserList, UserString
+from copy import deepcopy, copy
 
 
 __all__ = ['tag_object', 'get_tag']
@@ -63,6 +64,14 @@ class TaggedDict(Tagged, UserDict, dict):
                 self.data == other.data and
                 self._tag == other._tag)
 
+    def __deepcopy__(self, memo):
+        data_copy = deepcopy(self.data, memo)
+        return TaggedDict(data_copy, self._tag)
+
+    def __copy__(self):
+        data_copy = copy(self.data)
+        return TaggedDict(data_copy, self._tag)
+
 
 class TaggedList(Tagged, UserList, list):
     """
@@ -81,6 +90,14 @@ class TaggedList(Tagged, UserList, list):
                 self.data == other.data and
                 self._tag == other._tag)
 
+    def __deepcopy__(self, memo):
+        data_copy = deepcopy(self.data, memo)
+        return TaggedList(data_copy, self._tag)
+
+    def __copy__(self):
+        data_copy = copy(self.data)
+        return TaggedList(data_copy, self._tag)
+
 
 class TaggedString(Tagged, UserString, str):
     """
