@@ -1,14 +1,36 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 # -*- coding: utf-8 -*-
-
-
 from ...types import AsdfType
-from ...yamlutil import custom_tree_to_tagged_tree
 
 
-class AsdfObject(dict, AsdfType):
+class AsdfObject(dict):
+    pass
+
+from .constant import ConstantType
+from .ndarray import NDArrayType
+from .complex import ComplexType
+from .integer import IntegerType
+from .external_reference import ExternalArrayReference
+
+
+__all__ = ['AsdfObject', 'Software', 'HistoryEntry', 'ExtensionMetadata',
+    'SubclassMetadata', 'ConstantType', 'NDArrayType', 'ComplexType',
+    'IntegerType', 'ExternalArrayReference']
+
+
+class AsdfObjectType(AsdfType):
     name = 'core/asdf'
     version = '1.1.0'
+    supported_versions = {'1.0.0', '1.1.0'}
+    types = [AsdfObject]
+
+    @classmethod
+    def from_tree(cls, node, ctx):
+        return AsdfObject(node)
+
+    @classmethod
+    def to_tree(cls, data, ctx):
+        return dict(data)
 
 
 class Software(dict, AsdfType):
@@ -21,33 +43,19 @@ class HistoryEntry(dict, AsdfType):
     version = '1.0.0'
 
 
-class ExtensionMetadata(AsdfType):
+class ExtensionMetadata(dict, AsdfType):
     name = 'core/extension_metadata'
     version = '1.0.0'
 
-    def __init__(self, extension_class=None, software={}):
-        self.extension_class = extension_class
-        self.software = software
+    @property
+    def extension_class(self):
+        return self['extension_class']
 
-    @classmethod
-    def from_tree(cls, node, ctx):
-        return cls(**node)
-
-    @classmethod
-    def to_tree(cls, node, ctx):
-        tree = {}
-        tree['extension_class'] = node.extension_class
-        tree['software'] = custom_tree_to_tagged_tree(node.software, ctx)
-        return tree
+    @property
+    def software(self):
+        return self.get('software')
 
 
 class SubclassMetadata(dict, AsdfType):
     name = 'core/subclass_metadata'
     version = '1.0.0'
-
-
-from .constant import ConstantType
-from .ndarray import NDArrayType
-from .complex import ComplexType
-from .integer import IntegerType
-from .external_reference import ExternalArrayReference
