@@ -2,7 +2,6 @@
 # -*- coding: utf-8 -*-
 
 import sys
-import weakref
 
 import numpy as np
 from numpy import ma
@@ -12,7 +11,6 @@ from jsonschema import ValidationError
 from ...types import AsdfType
 from ... import schema
 from ... import util
-from ... import yamlutil
 
 
 _datatype_names = {
@@ -215,6 +213,7 @@ def numpy_array_to_list(array):
 
 class NDArrayType(AsdfType):
     name = 'core/ndarray'
+    version = '1.0.0'
     types = [np.ndarray, ma.MaskedArray]
 
     def __init__(self, source, shape, dtype, offset, strides,
@@ -222,7 +221,6 @@ class NDArrayType(AsdfType):
         self._asdffile = asdffile
         self._source = source
         self._block = None
-        self._block_data_weakref = None
         self._array = None
         self._mask = mask
 
@@ -251,20 +249,24 @@ class NDArrayType(AsdfType):
         # closed and replaced.  We need to check here and re-generate
         # the array if necessary, otherwise we risk segfaults when
         # memory mapping.
-        if self._block_data_weakref is not None:
-            block_data = self._block_data_weakref()
-            if block_data is not self.block.data:
+        if self._array is not None:
+            base = util.get_array_base(self._array)
+            if isinstance(base, np.memmap) and base._mmap is not None and base._mmap.closed:
                 self._array = None
-                self._block_data_weakref = None
 
         if self._array is None:
             block = self.block
             shape = self.get_actual_shape(
                 self._shape, self._strides, self._dtype, len(block))
+
+            if block.trust_data_dtype:
+                dtype = block.data.dtype
+            else:
+                dtype = self._dtype
+
             self._array = np.ndarray(
-                shape, self._dtype, block.data,
+                shape, dtype, block.data,
                 self._offset, self._strides, self._order)
-            self._block_data_weakref = weakref.ref(block.data)
             self._array = self._apply_mask(self._array, self._mask)
             if block.readonly:
                 self._array.setflags(write=False)
@@ -294,7 +296,7 @@ class NDArrayType(AsdfType):
             return "<{0} (unloaded) shape: {1} dtype: {2}>".format(
                 'array' if self._mask is None else 'masked array',
                 self._shape, self._dtype)
-        return repr(self._array)
+        return repr(self._make_array())
 
     def __str__(self):
         # str alone should not force loading of the data
@@ -302,7 +304,7 @@ class NDArrayType(AsdfType):
             return "<{0} (unloaded) shape: {1} dtype: {2}>".format(
                 'array' if self._mask is None else 'masked array',
                 self._shape, self._dtype)
-        return str(self._array)
+        return str(self._make_array())
 
     def get_actual_shape(self, shape, strides, dtype, block_size):
         """
@@ -343,13 +345,13 @@ class NDArrayType(AsdfType):
         if self._array is None:
             return self._dtype
         else:
-            return self._array.dtype
+            return self._make_array().dtype
 
     def __len__(self):
         if self._array is None:
             return self._shape[0]
         else:
-            return len(self._array)
+            return len(self._make_array())
 
     def __getattr__(self, attr):
         # We need to ignore __array_struct__, or unicode arrays end up
@@ -367,7 +369,6 @@ class NDArrayType(AsdfType):
             self._make_array().__setitem__(*args)
         except Exception as e:
             self._array = None
-            self._block_data_weakref = None
             raise e from None
 
     @classmethod
@@ -413,18 +414,18 @@ class NDArrayType(AsdfType):
 
     @classmethod
     def to_tree(cls, data, ctx):
+        if any(stride == 0 for stride in data.strides):
+            data = np.ascontiguousarray(data)
+
         base = util.get_array_base(data)
         shape = data.shape
         dtype = data.dtype
         offset = data.ctypes.data - base.ctypes.data
-        strides = None
 
-        if not data.flags.c_contiguous:
-            # We do not want to encode strides for broadcasted arrays
-            if not all(data.strides):
-                data = np.ascontiguousarray(data)
-            else:
-                strides = data.strides
+        if data.flags.c_contiguous:
+            strides = None
+        else:
+            strides = data.strides
 
         block = ctx.blocks.find_or_create_block_for_array(data, ctx)
 
@@ -441,8 +442,7 @@ class NDArrayType(AsdfType):
 
         if block.array_storage == 'inline':
             listdata = numpy_array_to_list(data)
-            result['data'] = yamlutil.custom_tree_to_tagged_tree(
-                listdata, ctx)
+            result['data'] = listdata
             result['datatype'] = dtype
         else:
             result['shape'] = list(shape)
@@ -463,8 +463,7 @@ class NDArrayType(AsdfType):
             if np.any(data.mask):
                 if block.array_storage == 'inline':
                     ctx.blocks.set_array_storage(ctx.blocks[data.mask], 'inline')
-                result['mask'] = yamlutil.custom_tree_to_tagged_tree(
-                    data.mask, ctx)
+                result['mask'] = data.mask
 
         return result
 
@@ -472,7 +471,9 @@ class NDArrayType(AsdfType):
     def _assert_equality(cls, old, new, func):
         if old.dtype.fields:
             if not new.dtype.fields:
-                assert False, "arrays not equal"
+                # This line is safe because this is actually a piece of test
+                # code, even though it lives in this file:
+                assert False, "arrays not equal" # nosec
             for a, b in zip(old, new):
                 cls._assert_equality(a, b, func)
         else:
@@ -485,7 +486,9 @@ class NDArrayType(AsdfType):
                     new = new.astype('U')
                 old = old.tolist()
                 new = new.tolist()
-                assert old == new
+                # This line is safe because this is actually a piece of test
+                # code, even though it lives in this file:
+                assert old == new # nosec
             else:
                 func(old, new)
 
