@@ -2,8 +2,9 @@
 
 import io
 import os
-import sys
 import warnings
+from contextlib import contextmanager
+from pathlib import Path
 
 try:
     from astropy.coordinates import ICRS
@@ -20,6 +21,8 @@ try:
 except ImportError:
     CartesianDifferential = None
 
+import yaml
+
 import asdf
 from ..asdf import AsdfFile, get_asdf_library_info
 from ..block import Block
@@ -27,6 +30,10 @@ from .httpserver import RangeHTTPServer
 from ..extension import default_extensions
 from ..exceptions import AsdfConversionWarning
 from .. import versioning
+from ..resolver import Resolver, ResolverChain
+from .. import generic_io
+from ..constants import YAML_TAG_PREFIX
+from ..versioning import AsdfVersion, get_version_map
 
 from ..tags.core import AsdfObject
 
@@ -36,30 +43,21 @@ except ImportError:
     INTERNET_OFF = False
 
 
-if sys.version_info >= (3, 7):
-    from importlib import resources
-else:
-    try:
-        import importlib_resources as resources
-    except ImportError:
-        resources = None
-
-
 __all__ = ['get_test_data_path', 'assert_tree_match', 'assert_roundtrip_tree',
            'yaml_to_asdf', 'get_file_sizes', 'display_warnings']
 
 
 def get_test_data_path(name, module=None):
-    if resources is None:
-        raise RuntimeError("The importlib_resources package is required to get"
-                           " test data on systems with Python < 3.7")
-
     if module is None:
         from . import data as test_data
         module = test_data
 
-    with resources.path(module, name) as path:
-        return str(path)
+    module_root = Path(module.__file__).parent
+
+    if name is None or name == "":
+        return str(module_root)
+    else:
+        return str(module_root/name)
 
 
 def assert_tree_match(old_tree, new_tree, ctx=None,
@@ -227,7 +225,7 @@ def _assert_roundtrip_tree(tree, tmpdir, *, asdf_check_func=None,
             asdf_check_func(ff)
 
     # Now try everything on an HTTP range server
-    if not INTERNET_OFF and not sys.platform.startswith('win'):
+    if not INTERNET_OFF:
         server = RangeHTTPServer()
         try:
             ff = AsdfFile(tree, extensions=extensions, **init_options)
@@ -290,13 +288,20 @@ def yaml_to_asdf(yaml_content, yaml_headers=True, standard_version=None):
     if standard_version is None:
         standard_version = versioning.default_version
 
+    standard_version = AsdfVersion(standard_version)
+
+    vm = get_version_map(standard_version)
+    file_format_version = vm["FILE_FORMAT"]
+    yaml_version = vm["YAML_VERSION"]
+    tree_version = vm["tags"]["tag:stsci.edu:asdf/core/asdf"]
+
     if yaml_headers:
         buff.write("""#ASDF {0}
 #ASDF_STANDARD {1}
-%YAML 1.1
+%YAML {2}
 %TAG ! tag:stsci.edu:asdf/
---- !core/asdf-{0}
-""".format(AsdfObject.version, standard_version).encode('ascii'))
+--- !core/asdf-{3}
+""".format(file_format_version, standard_version, yaml_version, tree_version).encode('ascii'))
     buff.write(yaml_content)
     if yaml_headers:
         buff.write(b"\n...\n")
@@ -352,3 +357,96 @@ def display_warnings(_warnings):
             warning.category.__name__,
             warning.message)
     return msg
+
+
+@contextmanager
+def assert_no_warnings(warning_class=None):
+    """
+    Assert that no warnings were emitted within the context.
+    Requires that pytest be installed.
+
+    Parameters
+    ----------
+    warning_class : type, optional
+        Assert only that no warnings of the specified class were
+        emitted.
+    """
+    import pytest
+    with pytest.warns(None) as recorded_warnings:
+        yield
+
+    if warning_class is not None:
+        assert not any(isinstance(w.message, warning_class) for w in recorded_warnings), \
+            display_warnings(recorded_warnings)
+    else:
+        assert len(recorded_warnings) == 0, display_warnings(recorded_warnings)
+
+
+def assert_extension_correctness(extension):
+    """
+    Assert that an ASDF extension's types are all correctly formed and
+    that the extension provides all of the required schemas.
+
+    Parameters
+    ----------
+    extension : asdf.AsdfExtension
+        The extension to validate
+    """
+    __tracebackhide__ = True
+
+    resolver = ResolverChain(
+        Resolver(extension.tag_mapping, "tag"),
+        Resolver(extension.url_mapping, "url"),
+    )
+
+    for extension_type in extension.types:
+        _assert_extension_type_correctness(extension, extension_type, resolver)
+
+
+def _assert_extension_type_correctness(extension, extension_type, resolver):
+    __tracebackhide__ = True
+
+    if extension_type.yaml_tag is not None and extension_type.yaml_tag.startswith(YAML_TAG_PREFIX):
+        return
+
+    if extension_type == asdf.stream.Stream:
+        # Stream is a special case.  It was implemented as a subclass of NDArrayType,
+        # but shares a tag with that class, so it isn't really a distinct type.
+        return
+
+    assert extension_type.name is not None, "{} must set the 'name' class attribute".format(extension_type.__name__)
+
+    # Currently ExtensionType sets a default version of 1.0.0,
+    # but we want to encourage an explicit version on the subclass.
+    assert "version" in extension_type.__dict__, "{} must set the 'version' class attribute".format(extension_type.__name__)
+
+    for check_type in extension_type.versioned_siblings + [extension_type]:
+        schema_location = resolver(check_type.yaml_tag)
+
+        assert schema_location is not None, (
+            "{} supports tag, {}, ".format(extension_type.__name__, check_type.yaml_tag) +
+            "but tag does not resolve.  Check the tag_mapping and uri_mapping " +
+            "properties on the related extension ({}).".format(extension_type.__name__)
+        )
+
+        try:
+            with generic_io.get_file(schema_location) as f:
+                schema = yaml.safe_load(f.read())
+        except Exception:
+            assert False, (
+                "{} supports tag, {}, ".format(extension_type.__name__, check_type.yaml_tag) +
+                "which resolves to schema at {}, but ".format(schema_location) +
+                "schema cannot be read."
+            )
+
+        assert "tag" in schema, (
+            "{} supports tag, {}, ".format(extension_type.__name__, check_type.yaml_tag) +
+            "but tag resolves to a schema at {} that is ".format(schema_location) +
+            "missing its tag field."
+        )
+
+        assert schema["tag"] == check_type.yaml_tag, (
+            "{} supports tag, {}, ".format(extension_type.__name__, check_type.yaml_tag) +
+            "but tag resolves to a schema at {} that ".format(schema_location) +
+            "describes a different tag: {}".format(schema["tag"])
+        )
