@@ -2,27 +2,22 @@
 # -*- coding: utf-8 -*-
 
 import io
-import os
-import re
-import warnings
 
 from jsonschema import ValidationError
-
-import yaml
-import pytest
-
 import numpy as np
 from numpy.testing import assert_array_equal
+import pytest
 
 import asdf
-from asdf import types
 from asdf import extension
 from asdf import resolver
 from asdf import schema
+from asdf import types
 from asdf import util
 from asdf import yamlutil
-
+from asdf import tagged
 from asdf.tests import helpers, CustomExtension
+from asdf.exceptions import AsdfWarning, AsdfConversionWarning
 
 
 class TagReferenceType(types.CustomType):
@@ -39,12 +34,12 @@ class TagReferenceType(types.CustomType):
     def from_tree(cls, tree, ctx):
         node = {}
         node['name'] = tree['name']
-        node['things'] = yamlutil.tagged_tree_to_custom_tree(tree['things'], ctx)
+        node['things'] = tree['things']
         return node
 
 
 def test_tagging_scalars():
-    astropy = pytest.importorskip('astropy', '3.0.0')
+    pytest.importorskip('astropy', '3.0.0')
     from astropy import units as u
 
     yaml = """
@@ -277,10 +272,9 @@ custom: !<tag:nowhere.org:custom/custom-1.0.0>
     # This should cause a warning but not an error because without explicitly
     # providing an extension, our custom type will not be recognized and will
     # simply be converted to a raw type.
-    with pytest.warns(None) as warning:
+    with pytest.warns(AsdfConversionWarning, match="tag:nowhere.org:custom/custom-1.0.0"):
         with asdf.open(buff):
             pass
-    assert len(warning) == 1
 
     buff.seek(0)
     with pytest.raises(ValidationError):
@@ -350,6 +344,32 @@ def test_default_check_in_schema():
     with pytest.raises(ValidationError):
         schema.check_schema(s)
 
+    schema.check_schema(s, validate_default=False)
+
+
+def test_check_complex_default():
+    default_software = tagged.TaggedDict(
+        {"name": "asdf", "version": "2.7.0"},
+        "tag:stsci.edu/asdf/core/software-1.0.0"
+    )
+
+    s = {
+        'type': 'object',
+        'properties': {
+            'a': {
+                'type': 'object',
+                'tag': 'tag:stsci.edu/asdf/core/software-1.0.0',
+                'default': default_software
+            }
+        }
+    }
+
+    schema.check_schema(s)
+
+    s['properties']['a']['tag'] = 'tag:stsci.edu/asdf/core/ndarray-1.0.0'
+    with pytest.raises(ValidationError):
+        schema.check_schema(s)
+
 
 def test_fill_and_remove_defaults():
     class DefaultType(dict, types.CustomType):
@@ -366,26 +386,82 @@ def test_fill_and_remove_defaults():
     yaml = """
 custom: !<tag:nowhere.org:custom/default-1.0.0>
   b: {}
+  d: {}
+  g: {}
+  j:
+    l: 362
     """
     buff = helpers.yaml_to_asdf(yaml)
     with asdf.open(buff, extensions=[DefaultTypeExtension()]) as ff:
         assert 'a' in ff.tree['custom']
         assert ff.tree['custom']['a'] == 42
         assert ff.tree['custom']['b']['c'] == 82
+        # allOf combiner should fill defaults from all subschemas:
+        assert ff.tree['custom']['d']['e'] == 122
+        assert ff.tree['custom']['d']['f'] == 162
+        # anyOf combiners should be ignored:
+        assert 'h' not in ff.tree['custom']['g']
+        assert 'i' not in ff.tree['custom']['g']
+        # oneOf combiners should be ignored:
+        assert 'k' not in ff.tree['custom']['j']
+        assert ff.tree['custom']['j']['l'] == 362
 
     buff.seek(0)
     with asdf.open(buff, extensions=[DefaultTypeExtension()],
                             do_not_fill_defaults=True) as ff:
         assert 'a' not in ff.tree['custom']
         assert 'c' not in ff.tree['custom']['b']
+        assert 'e' not in ff.tree['custom']['d']
+        assert 'f' not in ff.tree['custom']['d']
+        assert 'h' not in ff.tree['custom']['g']
+        assert 'i' not in ff.tree['custom']['g']
+        assert 'k' not in ff.tree['custom']['j']
+        assert ff.tree['custom']['j']['l'] == 362
         ff.fill_defaults()
         assert 'a' in ff.tree['custom']
         assert ff.tree['custom']['a'] == 42
         assert 'c' in ff.tree['custom']['b']
         assert ff.tree['custom']['b']['c'] == 82
+        assert ff.tree['custom']['b']['c'] == 82
+        assert ff.tree['custom']['d']['e'] == 122
+        assert ff.tree['custom']['d']['f'] == 162
+        assert 'h' not in ff.tree['custom']['g']
+        assert 'i' not in ff.tree['custom']['g']
+        assert 'k' not in ff.tree['custom']['j']
+        assert ff.tree['custom']['j']['l'] == 362
         ff.remove_defaults()
         assert 'a' not in ff.tree['custom']
         assert 'c' not in ff.tree['custom']['b']
+        assert 'e' not in ff.tree['custom']['d']
+        assert 'f' not in ff.tree['custom']['d']
+        assert 'h' not in ff.tree['custom']['g']
+        assert 'i' not in ff.tree['custom']['g']
+        assert 'k' not in ff.tree['custom']['j']
+        assert ff.tree['custom']['j']['l'] == 362
+
+
+def test_one_of():
+    """
+    Covers https://github.com/spacetelescope/asdf/issues/809
+    """
+    class OneOfType(dict, types.CustomType):
+        name = 'one_of'
+        organization = 'nowhere.org'
+        version = (1, 0, 0)
+        standard = 'custom'
+
+    class OneOfTypeExtension(CustomExtension):
+        @property
+        def types(self):
+            return [OneOfType]
+
+    yaml = """
+one_of: !<tag:nowhere.org:custom/one_of-1.0.0>
+  value: foo
+    """
+    buff = helpers.yaml_to_asdf(yaml)
+    with asdf.open(buff, extensions=[OneOfTypeExtension()]) as ff:
+        assert ff['one_of']['value'] == 'foo'
 
 
 def test_tag_reference_validation():
@@ -419,8 +495,8 @@ def test_foreign_tag_reference_validation():
         @classmethod
         def from_tree(cls, tree, ctx):
             node = {}
-            node['a'] = yamlutil.tagged_tree_to_custom_tree(tree['a'], ctx)
-            node['b'] = yamlutil.tagged_tree_to_custom_tree(tree['b'], ctx)
+            node['a'] = tree['a']
+            node['b'] = tree['b']
             return node
 
     class ForeignTypeExtension(CustomExtension):
@@ -515,16 +591,10 @@ def test_read_large_literal():
 
     buff = helpers.yaml_to_asdf(yaml)
 
-    with pytest.warns(UserWarning) as w:
+    with pytest.warns(AsdfWarning, match="Invalid integer literal value"):
         with asdf.open(buff) as af:
             assert af['integer'] == value
 
-        # We get two warnings: one for validation time, and one when defaults
-        # are filled. It seems like we could improve this architecture, though...
-        assert len(w) == 2
-        assert str(w[0].message).startswith('Invalid integer literal value')
-        assert str(w[1].message).startswith('Invalid integer literal value')
-
 
 def test_nested_array():
     s = {
@@ -603,7 +673,7 @@ properties:
 
 
 def test_type_missing_dependencies():
-    astropy = pytest.importorskip('astropy', '3.0.0')
+    pytest.importorskip('astropy', '3.0.0')
 
     class MissingType(types.CustomType):
         name = 'missing'
@@ -623,12 +693,10 @@ custom: !<tag:nowhere.org:custom/missing-1.1.0>
   b: {foo: 42}
     """
     buff = helpers.yaml_to_asdf(yaml)
-    with pytest.warns(None) as w:
+    with pytest.warns(AsdfConversionWarning, match="Failed to convert tag:nowhere.org:custom/missing-1.1.0"):
         with asdf.open(buff, extensions=[DefaultTypeExtension()]) as ff:
             assert ff.tree['custom']['b']['foo'] == 42
 
-    assert len(w) == 1
-
 
 def test_assert_roundtrip_with_extension(tmpdir):
     called_custom_assert_equal = [False]
@@ -655,18 +723,16 @@ def test_assert_roundtrip_with_extension(tmpdir):
     def check(ff):
         assert isinstance(ff.tree['custom'], CustomType)
 
-    with pytest.warns(None) as warnings:
+    with helpers.assert_no_warnings():
         helpers.assert_roundtrip_tree(
             tree, tmpdir, extensions=[CustomTypeExtension()])
 
-    assert len(warnings) == 0, helpers.display_warnings(warnings)
-
     assert called_custom_assert_equal[0] is True
 
 
 def test_custom_validation_bad(tmpdir):
     custom_schema_path = helpers.get_test_data_path('custom_schema.yaml')
-    asdf_file = os.path.join(str(tmpdir), 'out.asdf')
+    asdf_file = str(tmpdir.join('out.asdf'))
 
     # This tree does not conform to the custom schema
     tree = {'stuff': 42, 'other_stuff': 'hello'}
@@ -677,22 +743,22 @@ def test_custom_validation_bad(tmpdir):
 
     # Creating file using custom schema should fail
     with pytest.raises(ValidationError):
-        with asdf.AsdfFile(tree, custom_schema=custom_schema_path) as ff:
+        with asdf.AsdfFile(tree, custom_schema=custom_schema_path):
             pass
 
     # Opening file without custom schema should pass
-    with asdf.open(asdf_file) as ff:
+    with asdf.open(asdf_file):
         pass
 
     # Opening file with custom schema should fail
     with pytest.raises(ValidationError):
-        with asdf.open(asdf_file, custom_schema=custom_schema_path) as ff:
+        with asdf.open(asdf_file, custom_schema=custom_schema_path):
             pass
 
 
 def test_custom_validation_good(tmpdir):
     custom_schema_path = helpers.get_test_data_path('custom_schema.yaml')
-    asdf_file = os.path.join(str(tmpdir), 'out.asdf')
+    asdf_file = str(tmpdir.join('out.asdf'))
 
     # This tree conforms to the custom schema
     tree = {
@@ -703,7 +769,7 @@ def test_custom_validation_good(tmpdir):
     with asdf.AsdfFile(tree, custom_schema=custom_schema_path) as ff:
         ff.write_to(asdf_file)
 
-    with asdf.open(asdf_file, custom_schema=custom_schema_path) as ff:
+    with asdf.open(asdf_file, custom_schema=custom_schema_path):
         pass
 
 
@@ -716,7 +782,7 @@ def test_custom_validation_pathlib(tmpdir):
     from pathlib import Path
 
     custom_schema_path = Path(helpers.get_test_data_path('custom_schema.yaml'))
-    asdf_file = os.path.join(str(tmpdir), 'out.asdf')
+    asdf_file = str(tmpdir.join('out.asdf'))
 
     # This tree conforms to the custom schema
     tree = {
@@ -727,13 +793,13 @@ def test_custom_validation_pathlib(tmpdir):
     with asdf.AsdfFile(tree, custom_schema=custom_schema_path) as ff:
         ff.write_to(asdf_file)
 
-    with asdf.open(asdf_file, custom_schema=custom_schema_path) as ff:
+    with asdf.open(asdf_file, custom_schema=custom_schema_path):
         pass
 
 
 def test_custom_validation_with_definitions_good(tmpdir):
     custom_schema_path = helpers.get_test_data_path('custom_schema_definitions.yaml')
-    asdf_file = os.path.join(str(tmpdir), 'out.asdf')
+    asdf_file = str(tmpdir.join('out.asdf'))
 
     # This tree conforms to the custom schema
     tree = {
@@ -743,13 +809,13 @@ def test_custom_validation_with_definitions_good(tmpdir):
     with asdf.AsdfFile(tree, custom_schema=custom_schema_path) as ff:
         ff.write_to(asdf_file)
 
-    with asdf.open(asdf_file, custom_schema=custom_schema_path) as ff:
+    with asdf.open(asdf_file, custom_schema=custom_schema_path):
         pass
 
 
 def test_custom_validation_with_definitions_bad(tmpdir):
     custom_schema_path = helpers.get_test_data_path('custom_schema_definitions.yaml')
-    asdf_file = os.path.join(str(tmpdir), 'out.asdf')
+    asdf_file = str(tmpdir.join('out.asdf'))
 
     # This tree does NOT conform to the custom schema
     tree = {
@@ -762,19 +828,77 @@ def test_custom_validation_with_definitions_bad(tmpdir):
 
     # Creating file with custom schema should fail
     with pytest.raises(ValidationError):
-        with asdf.AsdfFile(tree, custom_schema=custom_schema_path) as ff:
+        with asdf.AsdfFile(tree, custom_schema=custom_schema_path):
             pass
 
     # Opening file without custom schema should pass
-    with asdf.open(asdf_file) as ff:
+    with asdf.open(asdf_file):
         pass
 
     # Opening file with custom schema should fail
     with pytest.raises(ValidationError):
-        with asdf.open(asdf_file, custom_schema=custom_schema_path) as ff:
+        with asdf.open(asdf_file, custom_schema=custom_schema_path):
             pass
 
 
+def test_custom_validation_with_external_ref_good(tmpdir):
+    custom_schema_path = helpers.get_test_data_path('custom_schema_external_ref.yaml')
+    asdf_file = str(tmpdir.join('out.asdf'))
+
+    # This tree conforms to the custom schema
+    tree = {
+        'foo': asdf.tags.core.Software(name="Microsoft Windows", version="95")
+    }
+
+    with asdf.AsdfFile(tree, custom_schema=custom_schema_path) as ff:
+        ff.write_to(asdf_file)
+
+    with asdf.open(asdf_file, custom_schema=custom_schema_path):
+        pass
+
+
+def test_custom_validation_with_external_ref_bad(tmpdir):
+    custom_schema_path = helpers.get_test_data_path('custom_schema_external_ref.yaml')
+    asdf_file = str(tmpdir.join('out.asdf'))
+
+    # This tree does not conform to the custom schema
+    tree = {
+        'foo': False
+    }
+
+    # Creating file without custom schema should pass
+    with asdf.AsdfFile(tree) as ff:
+        ff.write_to(asdf_file)
+
+    # Creating file with custom schema should fail
+    with pytest.raises(ValidationError):
+        with asdf.AsdfFile(tree, custom_schema=custom_schema_path):
+            pass
+
+    # Opening file without custom schema should pass
+    with asdf.open(asdf_file):
+        pass
+
+    # Opening file with custom schema should fail
+    with pytest.raises(ValidationError):
+        with asdf.open(asdf_file, custom_schema=custom_schema_path):
+            pass
+
+
+def test_load_custom_schema_deprecated():
+    custom_schema_path = helpers.get_test_data_path('custom_schema.yaml')
+
+    with pytest.deprecated_call():
+        schema.load_custom_schema(custom_schema_path)
+
+
+def test_load_schema_resolve_local_refs_deprecated():
+    custom_schema_path = helpers.get_test_data_path('custom_schema_definitions.yaml')
+
+    with pytest.deprecated_call():
+        schema.load_schema(custom_schema_path, resolve_local_refs=True)
+
+
 def test_nonexistent_tag(tmpdir):
     """
     This tests the case where a node is tagged with a type that apparently
@@ -796,16 +920,9 @@ a: !core/doesnt_exist-1.0.0
     """
 
     buff = helpers.yaml_to_asdf(yaml)
-    with pytest.warns(None) as w:
+    with pytest.warns(AsdfWarning, match="Unable to locate schema file"):
         with asdf.open(buff) as af:
             assert str(af['a']) == 'hello'
-            # Currently there are 3 warnings since one occurs on each of the
-            # validation passes. It would be good to consolidate these
-            # eventually
-            assert len(w) == 3, helpers.display_warnings(w)
-            assert str(w[0].message).startswith("Unable to locate schema file")
-            assert str(w[1].message).startswith("Unable to locate schema file")
-            assert str(w[2].message).startswith(af['a']._tag)
 
     # This is a more realistic case since we're using an external extension
     yaml = """
@@ -814,10 +931,76 @@ a: !<tag:nowhere.org:custom/doesnt_exist-1.0.0>
   """
 
     buff = helpers.yaml_to_asdf(yaml)
-    with pytest.warns(None) as w:
+    with pytest.warns(AsdfWarning, match="Unable to locate schema file"):
         with asdf.open(buff, extensions=CustomExtension()) as af:
             assert str(af['a']) == 'hello'
-            assert len(w) == 3, helpers.display_warnings(w)
-            assert str(w[0].message).startswith("Unable to locate schema file")
-            assert str(w[1].message).startswith("Unable to locate schema file")
-            assert str(w[2].message).startswith(af['a']._tag)
+
+
+@pytest.mark.parametrize("numpy_value,valid_types", [
+    (np.str_("foo"), {"string"}),
+    (np.bytes_("foo"), set()),
+    (np.float16(3.14), {"number"}),
+    (np.float32(3.14159), {"number"}),
+    (np.float64(3.14159), {"number"}),
+    # Evidently float128 is not available on Windows:
+    (getattr(np, "float128", np.float64)(3.14159), {"number"}),
+    (np.int8(42), {"number", "integer"}),
+    (np.int16(42), {"number", "integer"}),
+    (np.int32(42), {"number", "integer"}),
+    (np.longlong(42), {"number", "integer"}),
+    (np.uint8(42), {"number", "integer"}),
+    (np.uint16(42), {"number", "integer"}),
+    (np.uint32(42), {"number", "integer"}),
+    (np.uint64(42), {"number", "integer"}),
+    (np.ulonglong(42), {"number", "integer"}),
+])
+def test_numpy_scalar_type_validation(numpy_value, valid_types):
+    def _assert_validation(jsonschema_type, expected_valid):
+        validator = schema.get_validator()
+        try:
+            validator.validate(numpy_value, _schema={"type": jsonschema_type})
+        except ValidationError:
+            valid = False
+        else:
+            valid = True
+
+        if valid is not expected_valid:
+            if expected_valid:
+                description = "valid"
+            else:
+                description = "invalid"
+            assert False, "Expected numpy.{} to be {} against jsonschema type '{}'".format(
+                type(numpy_value).__name__, description, jsonschema_type
+            )
+
+    for jsonschema_type in valid_types:
+        _assert_validation(jsonschema_type, True)
+
+    invalid_types = {"string", "number", "integer", "boolean", "null", "object"} - valid_types
+    for jsonschema_type in invalid_types:
+        _assert_validation(jsonschema_type, False)
+
+
+def test_validator_visit_repeat_nodes():
+    ctx = asdf.AsdfFile()
+    node = asdf.tags.core.Software(name="Minesweeper")
+    tree = yamlutil.custom_tree_to_tagged_tree(
+        {"node": node, "other_node": node, "nested": {"node": node}},
+        ctx
+    )
+
+    visited_nodes = []
+    def _test_validator(validator, value, instance, schema):
+        visited_nodes.append(instance)
+
+    validator = schema.get_validator(ctx=ctx, validators=util.HashableDict(type=_test_validator))
+    validator.validate(tree)
+    assert len(visited_nodes) == 1
+
+    visited_nodes.clear()
+    validator = schema.get_validator(
+        validators=util.HashableDict(type=_test_validator),
+        _visit_repeat_nodes=True
+    )
+    validator.validate(tree)
+    assert len(visited_nodes) == 3
