@@ -14,6 +14,9 @@ import yaml
 import asdf
 from asdf import tagged
 from asdf import treeutil
+from asdf import yamlutil
+from asdf.compat.numpycompat import NUMPY_LT_1_14
+from asdf.exceptions import AsdfWarning
 
 from . import helpers
 
@@ -177,13 +180,13 @@ def test_implicit_conversion_warning():
         "val": nt(1, 2, np.ones(3))
     }
 
-    with pytest.warns(UserWarning, match="Failed to serialize instance"):
-        with asdf.AsdfFile(tree) as af:
+    with pytest.warns(AsdfWarning, match="Failed to serialize instance"):
+        with asdf.AsdfFile(tree):
             pass
 
-    with pytest.warns(None) as w:
-        with asdf.AsdfFile(tree, ignore_implicit_conversion=True) as af:
-            assert len(w) == 0
+    with helpers.assert_no_warnings():
+        with asdf.AsdfFile(tree, ignore_implicit_conversion=True):
+            pass
 
 
 @pytest.mark.xfail(reason='pyyaml has a bug and does not support tuple keys')
@@ -246,8 +249,7 @@ def test_yaml_internal_reference(tmpdir):
     }
 
     def check_yaml(content):
-        assert b'list:--&id002-*id002' in b''.join(content.split())
-
+        assert b'list:&id002-*id002' in b''.join(content.split())
     helpers.assert_roundtrip_tree(tree, tmpdir, raw_yaml_check_func=check_yaml)
 
 
@@ -275,3 +277,36 @@ def test_tag_object():
     tag = 'tag:nowhere.org:none/some/thing'
     instance = tagged.tag_object(tag, SomeObject())
     assert instance._tag == tag
+
+
+@pytest.mark.parametrize("numpy_value,expected_value", [
+    (np.str_("foo"), "foo"),
+    (np.bytes_("foo"), b"foo"),
+    (np.float16(3.14), 3.14),
+    (np.float32(3.14159), 3.14159),
+    (np.float64(3.14159), 3.14159),
+    # Evidently float128 is not available on Windows:
+    (getattr(np, "float128", np.float64)(3.14159), 3.14159),
+    (np.int8(42), 42),
+    (np.int16(42), 42),
+    (np.int32(42), 42),
+    (np.int64(42), 42),
+    (np.longlong(42), 42),
+    (np.uint8(42), 42),
+    (np.uint16(42), 42),
+    (np.uint32(42), 42),
+    (np.uint64(42), 42),
+    (np.ulonglong(42), 42),
+])
+def test_numpy_scalar(numpy_value, expected_value):
+    ctx = asdf.AsdfFile()
+    tree = {"value": numpy_value}
+    buffer = io.BytesIO()
+
+    yamlutil.dump_tree(tree, buffer, ctx)
+    buffer.seek(0)
+
+    if isinstance(expected_value, float) and NUMPY_LT_1_14:
+        assert yamlutil.load_tree(buffer)["value"] == pytest.approx(expected_value, rel=0.001)
+    else:
+        assert yamlutil.load_tree(buffer)["value"] == expected_value
