@@ -7,12 +7,12 @@ import warnings
 import importlib
 from collections import defaultdict
 
-import six
 from copy import copy
 
 from . import tagged
 from . import util
 from .versioning import AsdfVersion, AsdfSpec
+from .exceptions import AsdfWarning
 
 
 __all__ = ['format_tag', 'CustomType']
@@ -164,7 +164,8 @@ class AsdfTypeMeta(ExtensionTypeMeta):
         cls = super(AsdfTypeMeta, mcls).__new__(mcls, name, bases, attrs)
         # Classes using this metaclass get added to the list of built-in
         # extensions
-        _all_asdftypes.add(cls)
+        if name != "AsdfType":
+            _all_asdftypes.add(cls)
 
         return cls
 
@@ -293,7 +294,7 @@ class ExtensionType:
         Converts instances of custom types into tagged objects.
 
         It is more common for custom tag types to override `to_tree` instead of
-        this method. This method should only be overridden if it is necessary
+        this method. This method should be overridden if it is necessary
         to modify the YAML tag that will be used to tag this object.
 
         Parameters
@@ -311,7 +312,6 @@ class ExtensionType:
             An instance of `asdf.tagged.Tagged`.
         """
         obj = cls.to_tree(node, ctx)
-        yaml_tag = cls.yaml_tag
 
         node_cls = type(node)
         cls_name = node_cls.__name__
@@ -333,7 +333,8 @@ class ExtensionType:
                     "Failed to add subclass attribute(s) to node that is "
                     "not an object (is a {}). No subclass attributes are being "
                     "added (tag={}, subclass={})".format(
-                        type(obj).__name__, cls, node_cls)
+                        type(obj).__name__, cls, node_cls),
+                    AsdfWarning
                 )
 
         return tagged.tag_object(cls.yaml_tag, obj, ctx=ctx)
@@ -345,8 +346,11 @@ class ExtensionType:
 
         This method should be overridden by custom extension classes in order
         to define how custom types are deserialized from the YAML
-        representation back into their original types. The method will return
-        an instance of the original custom type.
+        representation back into their original types. Typically the method will
+        return an instance of the original custom type.  It is also permitted
+        to return a generator, which yields a partially constructed result, then
+        completes construction once the generator is drained.  This is useful
+        when constructing objects that contain reference cycles.
 
         This method is called as part of the process of reading an ASDF file in
         order to construct an `AsdfFile` object. Whenever a YAML subtree is
@@ -365,7 +369,8 @@ class ExtensionType:
 
         Returns
         -------
-            An instance of the custom type represented by this extension class.
+            An instance of the custom type represented by this extension class,
+            or a generator that yields that instance.
         """
         return cls(tree)
 
@@ -489,16 +494,14 @@ class ExtensionType:
         return AsdfSubclassProperty(attribute)
 
 
-@six.add_metaclass(AsdfTypeMeta)
-class AsdfType(ExtensionType):
+class AsdfType(ExtensionType, metaclass=AsdfTypeMeta):
     """
     Base class for all built-in ASDF types. Types that inherit this class will
     be automatically added to the list of built-ins. This should *not* be used
     for user-defined extensions.
     """
 
-@six.add_metaclass(ExtensionTypeMeta)
-class CustomType(ExtensionType):
+class CustomType(ExtensionType, metaclass=ExtensionTypeMeta):
     """
     Base class for all user-defined types.
     """
