@@ -51,7 +51,9 @@ def get_version_map(version):
             '{url_suffix}', 'asdf/version_map-{0}'.format(version))
         try:
             with generic_io.get_file(version_map_path, 'r') as fd:
-                version_map = yaml.load(
+                # The following call to yaml.load is safe because we're
+                # using a loader that inherits from pyyaml's SafeLoader.
+                version_map = yaml.load( # nosec
                     fd, Loader=_yaml_base_loader)
         except Exception:
             raise ValueError(
@@ -60,11 +62,11 @@ def get_version_map(version):
         # Separate the core tags from the rest of the standard for convenience
         version_map['core'] = {}
         version_map['standard'] = {}
-        for name, version in version_map['tags'].items():
-            if name.startswith('tag:stsci.edu:asdf/core'):
-                version_map['core'][name] = version
+        for tag_name, tag_version in version_map['tags'].items():
+            if tag_name.startswith('tag:stsci.edu:asdf/core'):
+                version_map['core'][tag_name] = tag_version
             else:
-                version_map['standard'][name] = version
+                version_map['standard'][tag_name] = tag_version
 
         _version_map[version] = version_map
 
@@ -164,7 +166,8 @@ supported_versions = [
     AsdfVersion('1.1.0'),
     AsdfVersion('1.2.0'),
     AsdfVersion('1.3.0'),
-    AsdfVersion('1.4.0')
+    AsdfVersion('1.4.0'),
+    AsdfVersion('1.5.0'),
 ]
 
 default_version = supported_versions[-1]
@@ -202,3 +205,8 @@ class VersionedMixin:
                 "Don't have information about version {0}".format(
                     self.version_string))
         return version_map
+
+
+# This is the ASDF Standard version at which the format of the history
+# field changed to include extension metadata.
+NEW_HISTORY_FORMAT_MIN_VERSION = AsdfVersion("1.2.0")
