@@ -18,7 +18,7 @@ from sphinx.util.nodes import set_source_info
 import asdf
 from asdf import AsdfFile
 from asdf.constants import ASDF_MAGIC, BLOCK_FLAG_STREAMED
-from asdf import versioning, util
+from asdf import versioning
 
 version_string = str(versioning.default_version)
 
@@ -49,7 +49,7 @@ class RunCodeDirective(Directive):
         try:
             try:
                 exec(code, GLOBALS)
-            except:
+            except Exception:
                 print(code)
                 raise
 
@@ -92,7 +92,7 @@ class AsdfDirective(Directive):
 
             with asdf.open(filename, **kwargs) as ff:
                 for i, block in enumerate(ff.blocks.internal_blocks):
-                    data = codecs.encode(block.data.tostring(), 'hex')
+                    data = codecs.encode(block.data.tobytes(), 'hex')
                     if len(data) > 40:
                         data = data[:40] + '...'.encode()
                     allocated = block._allocated
